#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "my_types.h"
#include "form.h"
#include "mirror.h"

double time_form(int flag, double sigma, double tau){
	double result = 1.0,
    	   C = 1.0,
    	   tau_small = 2.0/3.0*tau,
    	   x = sigma/tau;

	switch(flag){
		case 0:
			// cosinus**2 profile
			result = cos(M_PI/2*x)*cos(M_PI/2*x)*Heaviside(M_PI/2. - fabs(M_PI/2*x));
			// printf ("temp-form %lf\n", result);
			break;
		case 1:
			// gauss half_lenght
			C = 2.0*log(2.0);
			result = exp(-SQR(x)*C);
			break;
		case 2:
			// two pulses
			sigma = sigma + tau - tau_small*sqrt(log(10)/log(4));
			result = time_form( 1, sigma - 2*tau, tau) + sqrt(0.12)*time_form(1, sigma, tau_small);
			break;
		case 3:
			// two pulses: big first
			result = time_form( 1, sigma, tau) + sqrt(0.12)*time_form(1, sigma - 2*tau, tau_small);
			break;
		case 4:
			//gauss 6 half_length
			C = 32.0*log(2.0); // 2**5*log(2)
			result = exp(-C*SQR(x)*SQR(x)*SQR(x));
			break;
		case 5:
			//complex pulse
			result = (time_form( 1, sigma, tau/5.0) + time_form( 4, sigma, tau))*0.5;
			break;
		default:
			result = 1.0;
	}
	return result;
}

double sp_form(int flag, vect_t r, double r0){
	double result = 1.0,
		   z = r.z/r0,
		   a = 1.0,
		   C;
	vect_t *R_v;

	switch(flag){
		case 0:
			// flat-top
			result = a;
			break;
		case 1:
			// gauss (equal power,radius)
			C = log(10.0);
			a = 2.15678;
			result = a*exp(-SQR(z)*C);
			break;
		case 2:
			// gauss_4 (equal power,radius)
			C = 1.65872;
			a = 1.44082;
			result = a*exp(-SQR(z)*SQR(z)*C);
			break;
		case 3:
			// gauss_6 (equal power,radius)
			C = 1.38292;
			a = 1.26009;
			result = a*exp(-SQR(z)*SQR(z)*SQR(z)*C);
			break;
		case 4:
			 // complex spots
			R_v = (vect_t *)malloc(1 * sizeof(vect_t));
			R_v[0].x = r.x + 0.3*r0;
			R_v[0].y = r.y + 0.2*r0;
			result = sp_form(1, r, r0) + 3.*sp_form(1, R_v[0], 0.5*r0); // sp_form(0, r, r0) +
			free(R_v);
			break;
		case 5:
				//Laguerre-Gauss (l = 0, p = 1)
			C = 4.50761;
			a = 3.01766;
			z = C*SQR(z);
			result = a*exp(-z)*(1 - 2.0*z);
			break;
		default:
			result = 1.0;
	}

	return result;

}


