#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"
#include "my_types.h"
#include "mirror.h"
#include "solving_equations.h"
#include "record.h"

int main(int argn, char **argv) {

        //for MPI
    MPI_Init(&argn,&argv);  
    int size, rank;
    MPI_Comm_size(MPI_COMM_WORLD, &size);
    MPI_Comm_rank(MPI_COMM_WORLD, &rank);

    // if (rank == 0) 
    printf("Processes:%d\n",size);
    printf("Rank:%d\n",rank);

    double starttime, 
            endtime; 
    int start_point = 0,
        i/*, error*/;

    config_t cfg;
    int process_particles,  
        particle_number,  
        particle_amount;

        //reading configuration parameters
    read_config(&cfg);

    void (*field_definition)(vect_t, double, emf_t*, config_t) = NULL;
    void (*calculation)(Force_t*,particle_t*,config_t,void (*)(vect_t, double,emf_t*, config_t)) = NULL;

    	//field and scheme initialization + particle creation + reading of initial conditions
    field_definition = mirror; // for pondermotive task
    calculation = initialize_method(cfg.scheme,rank); // calculation scheme

    particle_t *el = malloc(cfg.steps * sizeof(particle_t));
    Force_t *F = malloc(cfg.steps * sizeof(Force_t));

    particle_t *el_ic = initial_conditions (&particle_amount);

		//particle distribution between processes
    process_particles = process_particle_amount (rank, size, particle_amount);

    	//calculation part
    starttime = MPI_Wtime();

    start_point = log_analizer( rank, size);	


    for(i = start_point; i < process_particles; i++) {
    	particle_number = PNUM(i, rank, size);
    	el[0] = el_ic[particle_number];

        calculation(F, el, cfg, field_definition);
        if (cfg.record) record_hdf5(el, cfg.steps, particle_number);
        else record_for_spectrum_old(el, cfg.steps, particle_number);

        printf("Pr_%d: Particle #%d was calculated. \n",rank,particle_number);
    }

    endtime = MPI_Wtime();

    printf("Pr_%d: Calculation was finished. \n",rank);
    printf("Time of working: %lf sec. \n",endtime-starttime);

    	//memory release
    free(F);
    free(el);
    free(el_ic);

    MPI_Finalize(); 

    return 0;
}


