#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <setjmp.h>
#include "my_types.h"
#include "integrate.h"
#include "mirror.h"
#include "form.h"

#define STEPS 100 //integration steps

// double ERRRE = 1.e-3; 
// double ERRAB = 1.e-3;

// jmp_buf myjmp;

double Heaviside(double x) {
    double result;

    result = (x>0.0) ? 1.0 : 0.0;

    return result;
}

struct params_set_int {
    double x1;
    double y1;
    double z1;
    double t;
    double phi0;
    double h;
    double f;
    double R;
    double a0;
    double radius;
    int sp_form;
    int flag; // 0 - without gsl, 1 - gsl
    int K;
};

void rotation (int flag, double phi, void *vect_1, void *vect_2) {
	vect_t *vect_init = (vect_t *) vect_1;
	vect_t *vect_end = (vect_t *) vect_2;

	(vect_end->x) =      (vect_init->x)*cos(phi) - flag*(vect_init->z)*sin(phi);
	(vect_end->z) = flag*(vect_init->x)*sin(phi) +      (vect_init->z)*cos(phi);
	(vect_end->y) = (vect_init->y);


}

double Int (double psi, void *p) {
    struct params_set_int *params = (struct params_set_int *) p;
    double x1 = (params -> x1);
    double y1 = (params -> y1);
    double z1 = (params -> z1);
    double h = (params -> h);
    double f = (params -> f);
    double R = (params -> R);
    double t = (params -> t);
    double a0 = (params -> a0);
    double phi0 = (params -> phi0);
    double r0 = (params -> radius);
    int K = (params -> K);

    double s, rSQ, l, z;
    double v;

    vect_t R_v;
    R_v.x = R*cos(psi);
    R_v.y = R * sin(psi);
    R_v.z = R; // for axial symmetrical case

	double x = R_v.x + h;
	double y = R_v.y;
	double E0x = a0;
	double E0y = 0.;

	E0x = E0x*sp_form(params -> sp_form, R_v, r0);

    s = (x*x + y*y)/(4*f*f);
	z = f*(s - 1);
	
	rSQ = sqrt((x1 - x)*(x1 - x) + (y1 - y)*(y1 - y) 
		+ (z1 - z)*(z1 - z));

	l = z - rSQ + t + phi0;	

	double S = sin(l);
	double C = cos(l);
	double u = 1.0/rSQ;

	switch (K) {
		case 0: //Ex
			v = ((2*f - x*(x - x1)*u)*S  + x*(x- x1)*u*u*C)*E0x*u;
			v += y*(x - x1)*(u*C - S)* E0y*u*u;
			break;
		case 1: //Ey
			v =  (-S + u*C)*(y - y1)*x*E0x*u*u;
			v += ((2*f - y*(y - y1)*u)*S + y*(y - y1)*u*u*C)*E0y*u;
			break;
		case 2: //Ez
			v = (x*E0x + y*E0y)*((1 -(z - z1)*u)*S + (z - z1)*u*u*C)*u;
			break;

		case 3: //iEx
			v = ((2*f - x*(x - x1)*u)*(-C) + x*(x- x1)*u*u*S)*E0x*u;
			v += y*(x - x1)*(u*S + C)* E0y*u*u;
			break;
		case 4: //iEy
			v =  (C + u*S)*(y - y1)*x*E0x*u*u;
			v += ((2*f - y*(y - y1)*u)*(-C) + y*(y - y1)*u*u*S)*E0y*u;
			break;
		case 5: //iEz
			v = (x*E0x + y*E0y)*((1 -(z - z1)*u)*(-C) + (z - z1)*u*u*S)*u;
			break;
		default:
			v = 0;
	}

	v *= R/2/M_PI/2/f;

    return v;
}


void mirror(vect_t R, double t, emf_t* Ff, config_t cfg){
	double phi = M_PI/180*cfg.phigr;
	double r = (2.0*M_PI)/(0.8*1.e-6)*cfg.ro0; // meters
	double fef,f,h;
	double sigma;
	vect_t v1, v;
	emf_t F;
	
	v = R;
	// v.y = y;
	// v.z = z;
	fef = 2*cfg.fnumber*r;
	f = fef *(1 + cos(phi))/2.0;
	h = (2*f*sin(phi))/(1 + cos(phi));
	rotation(1,phi,&v,&v1);

	sigma = t - v.z + cfg.zf - cfg.tau;

    struct params_set_int params_ext;

	params_ext.x1 = v1.x;
    params_ext.y1 = v1.y;
    params_ext.z1 = v1.z;
    params_ext.h = h;
    params_ext.f = f;
    params_ext.t = t;
    params_ext.a0 = cfg.a0;
    params_ext.phi0 = cfg.fi0;
    params_ext.radius = r;
    params_ext.sp_form = cfg.space_form;

	// else 
	core(&F, (void *)&params_ext, cfg, sigma);

    rotation(-1, phi, &F, Ff);
    rotation(-1, phi, &(F.iEx), &(Ff->iEx));

    // printf ("Ex = %lf\n", Ff->Ex);
}


void core(emf_t* F, void* p, config_t cfg, double sigma){
	struct params_set_int *params = (struct params_set_int *) p;
	double r = (params -> radius);
	int i;
	double time_form_now = 1.0;
	// time_form(cfg.time_form, sigma, cfg.tau);

    for (i = 0; i < 6; i++) {
    	params[0].K = i;
    	*(&(F[0].Ex) + i) = time_form_now*simpson( 0.0, r, &Ext, (void *)params, STEPS);
    }

}

double Ext (double R, void *p) {
    struct params_set_int *params = (struct params_set_int *) p;
    (params->R) = R;
    double result;

    result = simpson( 0.0, 2.*M_PI,  &Int, (void *)params, STEPS);

    return result;
}
