#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <setjmp.h>
#include "my_types.h"
#include "record.h"
#include "hdf5.h"

jmp_buf myjmp;

FILE *file_open_read(char *name){

    FILE *result;

    if (fopen(name, "r") != NULL) result = fopen(name, "r");
    else { printf("\n\n     Please, make a file: %s\n\n\n",name); abort();}

    return result;

}

int read_config(config_t* cfg)
{
    FILE *file;
    // double l0;

    file = file_open_read("config/parameters.cfg");

/*    if (fopen("config//parameters.cfg", "r") != NULL) file = fopen("config//parameters.cfg", "r");
    else { printf("Please, make a file: config//parameters.cfg\n"); return 1;}*/

    fscanf(file,"%*s %*c %d\n",&cfg->record);
    fscanf(file,"%*s %*c %d\n",&cfg->scheme);
    // fscanf(file,"%*s %*c %d\n",&cfg->mode);
    // fscanf(file,"%*s %*c %d\n",&cfg->q);
    fscanf(file,"%*s %*c %d\n",&cfg->steps);
    fscanf(file,"%*s %*c %lf\n",&cfg->dt);
    fscanf(file, "%*[^\n]");
    fscanf(file, "%*[^\n]");
    fscanf(file,"%*s %*c %lf\n",&cfg->a0);
    fscanf(file,"%*s %*c %lf\n",&cfg->tau);
    fscanf(file,"%*s %*c %lf\n",&cfg->zf);
    fscanf(file,"%*s %*c %lf\n",&cfg->fi0);
    fscanf(file,"%*s %*c %lf\n",&cfg->fnumber);
    fscanf(file,"%*s %*c %d\n",&cfg->time_form);
    fscanf(file,"%*s %*c %d\n",&cfg->space_form);
    fscanf(file, "%*[^\n]");
    fscanf(file, "%*[^\n]");
    // fscanf(file,"%*s %*c %lf\n",&cfg->e0);
    // fscanf(file, "%*[^\n]");
    // fscanf(file, "%*[^\n]");
    fscanf(file,"%*s %*c %lf\n",&cfg->ro0);
    fscanf(file,"%*s %*c %lf\n",&cfg->phigr);
    fclose(file);

    return 0;
}

int record(particle_t* p, int num_of_parts,int n) {
    FILE *file;
      
    char file_name[28];
    int i;
    
    sprintf(file_name, "results//particle%d.dat", n);
    file = fopen(file_name, "w");
    
    fprintf(file,"time x y z px py pz\n");
          
    for (i = 0; i < num_of_parts; i++) {
        fprintf(file,"%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf\n",(p+i)->time,(p+i)->x,(p+i)->y,(p+i)->z,
            (p+i)->px,(p+i)->py,(p+i)->pz);
    }
    
    fclose(file);
    return num_of_parts;
}

void hdf5_attr_str_to_object(hid_t object_id, char *attr_name, char *str) {
    hid_t attr, space, type_id, memtype;
    herr_t err;

    type_id = H5Tcopy(H5T_C_S1);
    err = H5Tset_size(type_id, strlen(str));
    memtype = H5Tcopy(H5T_C_S1);
    err = H5Tset_size(memtype, strlen(str));
    space = H5Screate(H5S_SCALAR);
    attr = H5Acreate2(object_id, attr_name, type_id, space, H5P_DEFAULT, H5P_DEFAULT);
    err = H5Awrite(attr, memtype, str);

    err = H5Aclose(attr);
    err = H5Sclose(space);
    err = H5Tclose(memtype);
    err = H5Tclose(type_id);
} 

void hdf5_sruct_array_record (char* array_name, int stuct_size, int element_ammount, void* array, char* attributes, hid_t* file) {

    hid_t space, dset;
    herr_t status;

    hsize_t dims[2];
    dims[0] = element_ammount;
    dims[1] = stuct_size;
  
    char *dset_name = array_name;
    space = H5Screate_simple(2, dims, NULL);
    dset = H5Dcreate(*file, dset_name, H5T_NATIVE_DOUBLE, space, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
    status = H5Dwrite(dset, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, array);
    hdf5_attr_str_to_object(dset, "description", attributes);

    status = H5Sclose(space);
    status = H5Dclose (dset);

}


int record_hdf5(particle_t* p, int num_of_parts, int n) {

    char file_name[28];
    sprintf(file_name, "results/particle%d.h5", n);
    hid_t file;
    herr_t status;

    file = H5Fcreate (file_name, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);

    hdf5_sruct_array_record("coordinates", sizeof(particle_t)/8, num_of_parts, p, "px py pz x y z time", &file);
    // hdf5_sruct_array_record("acceleration", sizeof(vect_t)/8, num_of_parts, w, "wx wy wz", &file);

    status = H5Fclose (file);
    
    return num_of_parts;
}

int process_particle_amount (int rank, int processes, int particle_amount) {
    int result = 0;

    for(;;) {
        if ((particle_amount - (rank + 1) - processes*result) < 0) break;
        else result++;
    }

    return result;
}

particle_t* initial_conditions (int* particle_amount/*, int* error*/) {

    FILE *file;

    file = file_open_read("config/particle.cfg");

    // if (fopen("config//particle.cfg", "r") != NULL) file = fopen("config//particle.cfg", "r");
    // else { printf("Please, make a file: config//particle.cfg\n"); *error = 1; return (particle_t*) error;}
    fscanf(file,"%*s %*c %d\n",particle_amount);
    fscanf(file,"%*s %*s %*s %*s %*s %*s\n");
    particle_t *el_ic = malloc(*particle_amount * sizeof(particle_t));

    int i;
    for(i = 0;i < *particle_amount; i++) {
        fscanf(file,"%lf %lf %lf %lf %lf %lf\n",&el_ic[i].px,&el_ic[i].py,&el_ic[i].pz,&el_ic[i].x,&el_ic[i].y,&el_ic[i].z);
        el_ic[i].time = 0;
    }

    fclose(file);

    return el_ic;
}


int record_for_spectrum_old(particle_t* p, int num_of_parts,int n) {
    FILE *file_x;
    FILE *file_y;
    FILE *file_z;
    FILE *file_px;
    FILE *file_py;
    FILE *file_pz;
    // FILE *file_wx;
    // FILE *file_wy;
    // FILE *file_wz;
    FILE *file_time;

    int i;
    char c[28];

    sprintf(c, "results/x%d.dat", n );
    file_x = fopen(c, "w");

    sprintf(c, "results/y%d.dat", n);
    file_y = fopen(c, "w");

    sprintf(c, "results/z%d.dat", n);
    file_z = fopen(c, "w");

    sprintf(c, "results/px%d.dat", n);
    file_px = fopen(c, "w");

    sprintf(c, "results/py%d.dat", n);
    file_py = fopen(c, "w");

    sprintf(c, "results/pz%d.dat", n);
    file_pz = fopen(c, "w");

    // sprintf(c, "results/wx%d.dat", n);
    // file_wx = fopen(c, "w");

    // sprintf(c, "results/wy%d.dat", n);
    // file_wy = fopen(c, "w");
    
    // sprintf(c, "results/wz%d.dat", n);
    // file_wz = fopen(c, "w");

    sprintf(c, "results/t%d.dat", n);
    file_time = fopen(c, "w");

    for (i = 0; i < num_of_parts; i++) {
        fprintf(file_x,"%lf\n",(p+i)->x);
        fprintf(file_y,"%lf\n",(p+i)->y);
        fprintf(file_z,"%lf\n",(p+i)->z);
        fprintf(file_px,"%lf\n",(p+i)->px);
        fprintf(file_py,"%lf\n",(p+i)->py);
        fprintf(file_pz,"%lf\n",(p+i)->pz);
        // fprintf(file_wx,"%lf\n",(w+i)->x);
        // fprintf(file_wy,"%lf\n",(w+i)->y);
        // fprintf(file_wz,"%lf\n",(w+i)->z);
        fprintf(file_time,"%lf\n",(p+i)->time);
    }

    fclose(file_x);
    fclose(file_y);
    fclose(file_z);
    fclose(file_px);
    fclose(file_py);
    fclose(file_pz);
    fclose(file_time);
    // fclose(file_wx);
    // fclose(file_wy);
    // fclose(file_wz);

    return i;
}

void line_skip(FILE *file) {
    int c = getc(file);
    while (c != '\n')
        c = getc(file);
}


void newsigabort() {

    printf("I have started from the beginning.\n");

    longjmp(myjmp, 1);

}


int log_analizer(int rank, int size){
    int point = 0,
        last_part = 0,
        r_writer,
        wr_partic;

    // printf("3\n");

    FILE *file;

    signal(SIGABRT, newsigabort);

    if(!setjmp(myjmp))
        file = file_open_read("log.in"); // do normally
    else {
        return 0;
        // printf("1\n");
    }
    // file = file_open_read("log.in");

    while (getc(file) != EOF){
        if (!fscanf(file,"%*c%*c%d%*c %*s %*c%d %*s %*s \n", &r_writer, &wr_partic))
            line_skip(file);
        if (r_writer == rank) {last_part = wr_partic;}
    }
    fclose(file);

    point = PNEXT(last_part, rank, size);

    // printf ("%d\n",last_part);

    if (last_part) printf ("I am #%d and I finished at particle #%d last time, start point - %d.\n", rank, last_part, point);

    return point;
}
